@@ -1,5 +1,5 @@
 
-/*	$Id: tixImgCmp.c,v 1.3.2.1 2002/12/15 04:21:53 idiscovery Exp $	*/
+/*	$Id: tixImgCmp.c,v 1.6 2004/03/28 02:44:56 hobbs Exp $	*/
 
 /* 
  * tkImgCmp.c --
@@ -17,16 +17,6 @@
 #include <tixInt.h>
 #include <tixDef.h>
 
-/* Should use USE_OLD_IMAGE ImgCmpCreate */
-#if (TCL_MAJOR_VERSION <= 7)
-#  define USE_OLD_IMAGE
-#else
-#  if ((TCL_MAJOR_VERSION == 8) && (TCL_MINOR_VERSION <= 2))
-#  define USE_OLD_IMAGE
-#  endif
-   /* Otherwise use tk.h's definition */
-#endif
-
 /*
  * ToDo: 
  *	- lineconfig and itemconfig command
@@ -152,7 +142,7 @@ typedef union CmpItemPtr {
  * The type record for bitmap images:
  */
 static int		ImgCmpCreate _ANSI_ARGS_((Tcl_Interp *interp,
-			    char *name, int argc, char **argv,
+			    char *name, int argc, Tcl_Obj *CONST objv[],
 			    Tk_ImageType *typePtr, Tk_ImageMaster master,
 			    ClientData *clientDataPtr));
 static ClientData	ImgCmpGet _ANSI_ARGS_((Tk_Window tkwin,
@@ -167,15 +157,15 @@ static void		ImgCmpDelete _ANSI_ARGS_((ClientData clientData));
 static void		ImgCmpFreeResources _ANSI_ARGS_((ClientData clientData));
 
 Tk_ImageType tixCompoundImageType = {
-    "compound",			/* name */
-    (Tk_ImageCreateProc *) ImgCmpCreate,		/* createProc */
-    ImgCmpGet,			/* getProc */
-    ImgCmpDisplay,		/* displayProc */
-    ImgCmpFree,			/* freeProc */
-    ImgCmpDelete,		/* deleteProc */
-    (Tk_ImagePostscriptProc *) NULL, /* postscriptProc */
-    (Tk_ImageType *) NULL,	/* nextPtr */
-    (char *) NULL, 		/* reserved */
+    "compound",                     /* name */
+    ImgCmpCreate,                   /* createProc */
+    ImgCmpGet,                      /* getProc */
+    ImgCmpDisplay,                  /* displayProc */
+    ImgCmpFree,                     /* freeProc */
+    ImgCmpDelete,                   /* deleteProc */
+    NULL,                           /* postscriptProc (tk8.3 or later)*/
+    NULL,                           /* nextPtr */
+    NULL,                           /* reserved */
 };
 
 static Tk_ConfigSpec configSpecs[] = {
@@ -334,26 +324,26 @@ static Tk_ConfigSpec textConfigSpecs[] = {
  */
 
 static int		ImgCmpCmd _ANSI_ARGS_((ClientData clientData,
-			    Tcl_Interp *interp, int argc, char **argv));
+			    Tcl_Interp *interp, int argc, CONST84 char **argv));
 static void		ImgCmpCmdDeletedProc _ANSI_ARGS_((
 			    ClientData clientData));
 static int		ImgCmpConfigureMaster _ANSI_ARGS_((
-			    CmpMaster *masterPtr, int argc, char **argv,
+			    CmpMaster *masterPtr, int argc, CONST84 char **argv,
 			    int flags));
 CmpBitmapItem *		AddNewBitmap _ANSI_ARGS_((CmpMaster *masterPtr,
 			    CmpLine *line,
-			    int argc, char **argv));
+			    int argc, CONST84 char **argv));
 CmpImageItem * 		AddNewImage _ANSI_ARGS_((CmpMaster *masterPtr,
 			    CmpLine *line,
-			    int argc, char **argv));
+			    int argc, CONST84 char **argv));
 CmpSpaceItem * 		AddNewSpace _ANSI_ARGS_((CmpMaster *masterPtr,
 			    CmpLine *line,
-			    int argc, char **argv));
+			    int argc, CONST84 char **argv));
 CmpTextItem * 		AddNewText _ANSI_ARGS_((CmpMaster *masterPtr,
 			    CmpLine *line,
-			    int argc, char **argv));
+			    int argc, CONST84 char **argv));
 CmpLine* 		AddNewLine _ANSI_ARGS_((CmpMaster *masterPtr,
-			    int argc, char **argv));
+			    int argc, CONST84 char **argv));
 static void		CalculateMasterSize _ANSI_ARGS_((
 			    ClientData clientData));
 static void		ChangeImageWhenIdle _ANSI_ARGS_((
@@ -390,7 +380,7 @@ ImgCmpCreate(interp, name, argc, objv, typePtr, master, clientDataPtr)
 				 * image. */
     char *name;			/* Name to use for image. */
     int argc;			/* Number of arguments. */
-    char **objv;		/* Argument strings for options (doesn't
+    Tcl_Obj *CONST objv[];	/* Argument strings for options (doesn't
 				 * include image name or type). */
     Tk_ImageType *typePtr;	/* Pointer to our type record (not used). */
     Tk_ImageMaster master;	/* Token for image, to be used by us in
@@ -400,17 +390,22 @@ ImgCmpCreate(interp, name, argc, objv, typePtr, master, clientDataPtr)
 {
     CmpMaster *masterPtr;
     int i;
-    char *argvbuf[10];
-    char **argv = argvbuf;
+    CONST84 char *argvbuf[10];
+    CONST84 char **argv = argvbuf;
 
     /*
      * Convert the objv arguments into string equivalent.
      */
     if (argc > 10) {
-	argv = (char **) ckalloc(argc * sizeof(char *));
+	argv = (CONST84 char **) ckalloc(argc * sizeof(char *));
     }
     for (i = 0; i < argc; i++) {
-	argv[i] = TixGetStringFromObj(objv[i], NULL);
+        /*
+         * no need to free the value returned by Tcl_GetString. It's
+         * managed by Tcl's object system.
+         */
+
+	argv[i] = Tcl_GetString(objv[i]);
     }
 
     masterPtr = (CmpMaster *) ckalloc(sizeof(CmpMaster));
@@ -474,7 +469,7 @@ ImgCmpConfigureMaster(masterPtr, argc, argv, flags)
     CmpMaster *masterPtr;	/* Pointer to data structure describing
 				 * overall bitmap image to (reconfigure). */
     int argc;			/* Number of entries in argv. */
-    char **argv;		/* Pairs of configuration options for image. */
+    CONST84 char **argv;	/* Pairs of configuration options for image. */
     int flags;			/* Flags to pass to Tk_ConfigureWidget,
 				 * such as TK_CONFIG_ARGV_ONLY. */
 {
@@ -553,7 +548,7 @@ ImgCmpCmd(clientData, interp, argc, argv)
     ClientData clientData;	/* Information about button widget. */
     Tcl_Interp *interp;		/* Current interpreter. */
     int argc;			/* Number of arguments. */
-    char **argv;		/* Argument strings. */
+    CONST84 char **argv;	/* Argument strings. */
 {
     CmpMaster *masterPtr = (CmpMaster *) clientData;
     int c, code;
@@ -692,7 +687,7 @@ CmpLine *
 AddNewLine(masterPtr, argc, argv)
     CmpMaster *masterPtr;
     int argc;			/* Number of arguments. */
-    char **argv;		/* Argument strings. */
+    CONST84 char **argv;	/* Argument strings. */
 {
     CmpLine * lPtr = (CmpLine *)ckalloc(sizeof(CmpLine));
 
@@ -737,7 +732,7 @@ AddNewBitmap(masterPtr, line, argc, argv)
     CmpMaster *masterPtr;
     CmpLine *line;
     int argc;			/* Number of arguments. */
-    char **argv;		/* Argument strings. */
+    CONST84 char **argv;	/* Argument strings. */
 {
     CmpItemPtr p;
     XGCValues gcValues;
@@ -797,7 +792,7 @@ AddNewImage(masterPtr, line, argc, argv)
     CmpMaster *masterPtr;
     CmpLine *line;
     int argc;			/* Number of arguments. */
-    char **argv;		/* Argument strings. */
+    CONST84 char **argv;	/* Argument strings. */
 {
     CmpItemPtr p;
 
@@ -848,7 +843,7 @@ AddNewSpace(masterPtr, line, argc, argv)
     CmpMaster *masterPtr;
     CmpLine *line;
     int argc;			/* Number of arguments. */
-    char **argv;		/* Argument strings. */
+    CONST84 char **argv;	/* Argument strings. */
 {
     CmpItemPtr p;
 
@@ -886,7 +881,7 @@ AddNewText(masterPtr, line, argc, argv)
     CmpMaster *masterPtr;
     CmpLine *line;
     int argc;			/* Number of arguments. */
-    char **argv;		/* Argument strings. */
+    CONST84 char **argv;	/* Argument strings. */
 {
     CmpItemPtr p;
     XGCValues gcValues;
@@ -1205,8 +1200,8 @@ ImgCmpDisplay(clientData, display, drawable, imageX, imageY, width,
 	      case TYPE_BITMAP:
 		XCopyPlane(Tk_Display(masterPtr->tkwin), p.bitmap->bitmap,
 		    drawable, p.bitmap->gc, 0, 0,
-		    p.bitmap->width  - 2*p.item->padX,
-		    p.bitmap->height - 2*p.item->padY,
+		    (unsigned) p.bitmap->width  - 2*p.item->padX,
+		    (unsigned) p.bitmap->height - 2*p.item->padY,
 		    dx, dy+extraY, 1);
 
 		break;
@@ -1356,43 +1351,43 @@ ImgCmpFreeResources(masterData)
     Tk_Preserve((ClientData) masterPtr);
 
     if (masterPtr->isDeleted == 0) {
+        masterPtr->isDeleted = 1;
 
-	    masterPtr->isDeleted = 1;
-
-	    Tk_DeleteEventHandler(masterPtr->tkwin,
-				  StructureNotifyMask, 
-				  CmpEventProc, (ClientData)masterPtr);
-	    for (lPtr=masterPtr->lineHead; lPtr;) {
-		    CmpLine * toDelete = lPtr;
-		    lPtr = lPtr->next;
+        Tk_DeleteEventHandler(masterPtr->tkwin,
+		StructureNotifyMask, 
+		CmpEventProc, (ClientData)masterPtr);
+        for (lPtr=masterPtr->lineHead; lPtr;) {
+            CmpLine * toDelete = lPtr;
+            lPtr = lPtr->next;
 		    
-		    for (p.item=toDelete->itemHead; p.item;) {
-			    CmpItemPtr toDelete;
+            for (p.item=toDelete->itemHead; p.item;) {
+                CmpItemPtr toDelete;
 			    
-			    toDelete.item = p.item;
-			    p.item=p.item->next;
+                toDelete.item = p.item;
+                p.item=p.item->next;
 			    
-			    FreeItem(toDelete);
-		    }
-		    FreeLine(toDelete);
-	    }
+                FreeItem(toDelete);
+            }
+            FreeLine(toDelete);
+        }
 
-	    if (masterPtr->changing) {
-		    Tk_CancelIdleCall(CalculateMasterSize, (ClientData)masterPtr);
-	    }
-	    masterPtr->tkMaster = NULL;
+        if (masterPtr->changing) {
+            Tk_CancelIdleCall(CalculateMasterSize, (ClientData)masterPtr);
+        }
+        masterPtr->tkMaster = NULL;
 
-	    if (masterPtr->imageCmd != NULL) {
-		    char * cmd = Tcl_GetCommandName(masterPtr->interp,masterPtr->imageCmd);
-		    masterPtr->imageCmd = NULL;
-		    Tcl_DeleteCommand(masterPtr->interp, cmd);
-	    }
+        if (masterPtr->imageCmd != NULL) {
+            CONST84 char * cmd = Tcl_GetCommandName(masterPtr->interp,
+                masterPtr->imageCmd);
+            masterPtr->imageCmd = NULL;
+            Tcl_DeleteCommand(masterPtr->interp, cmd);
+        }
 
-	    if (masterPtr->gc != None) {
-		    Tk_FreeGC(masterPtr->display, masterPtr->gc);
-	    }
+        if (masterPtr->gc != None) {
+            Tk_FreeGC(masterPtr->display, masterPtr->gc);
+        }
 	    
-	    Tk_FreeOptions(configSpecs, (char *) masterPtr, masterPtr->display, 0);
+        Tk_FreeOptions(configSpecs, (char *) masterPtr, masterPtr->display, 0);
     }
 
     Tk_Release((ClientData) masterPtr);
