@@ -1,5 +1,5 @@
 
-/*	$Id: tixUnixPort.h,v 1.1.1.1 2000/05/17 11:08:54 idiscovery Exp $	*/
+/*	$Id: tixUnixPort.h,v 1.2 2001/01/15 10:35:57 ioilam Exp $	*/
 
 /*
  * tixUnixPort.h --
@@ -21,6 +21,8 @@
 struct _TixpSubRegion {
     XRectangle rect;
     int rectUsed;
+    int origX;
+    int origY;
 };
 
 #ifdef UCHAR_SUPPORTED
